/*
 * The SmartWeb Framework Maven Plugin
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework Maven Plugin please visit
 *
 *                        http://smartweb.sourceforge.net/maven
 */
package net.smartlab.web.maven;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Bootstrapping is the process by which you load a very small and very simple
 * pure java program with no dependencies that, in turn, loads, configures, and
 * runs more complex programs with varying dependencies. Bootstrapping lets you
 * run your container without polluting the system classpath. This allows you to
 * run your deployed applications with the unpolluted system classpath as its
 * parent. You’ve achieved classloader isolation. When would you want to
 * bootstrap? Any time you want an unpolluted system classpath, which I’m
 * finding is often convenient.
 * 
 * @author gcontartese
 */
public class Booter {

	protected Log logger = LogFactory.getLog(Booter.class);


	/**
	 * Return a new ClassLoader ,parent of SystemClassLoader. It can be used to
	 * run a main class completed isolated.
	 * 
	 * @param classesFile Set containing all <code>Class</code> as java.io.File
	 *        . Be carefull any file it dependency of OperationSystem
	 * @param classpath Classpath of project as java.io.File
	 * @return an isolated a new ClassLoader from System (parent class loader).
	 * @throws Exception if any MalformedUriException is thrown will parsing
	 *         files
	 */
	public synchronized ClassLoader getClassLoader(Set classesFile, List classpath) throws Exception {
		// assign all classes in classpath
		List urls = new ArrayList();
		for (Iterator it = classesFile.iterator(); it.hasNext();) {
			File f = (File)it.next();
			if (!f.isDirectory()) {
				continue;
			}
			urls.add(f.toURI().toURL());
		}
		Iterator it = classpath.iterator();
		// assign all libraries
		while (it.hasNext()) {
			File lib = new File((String)it.next());
			urls.add(lib.toURI().toURL());
		}
		// feed your URLs to a URLClassLoader!
		ClassLoader classloader = new URLClassLoader((URL[])urls.toArray(new URL[0]), ClassLoader
				.getSystemClassLoader().getParent());
		it = urls.iterator();
		return classloader;
	}
}
